/*
 *  Licensed Materials - Property of IBM
 *  
 *  CICS SupportPac CA1T
 * 
 *   Copyright IBM Corporation 2008, 2011
 *  All Rights Reserved
 *  
 *  US Government Users Restricted Rights  Use,
 *  duplication, or disclosure restricted by GSA ADP
 *  Schedule Contract with IBM Corporation
 */

/*
 *      File Name:    FailOverExit.java
 *      Version:      CA1T V2.1
 *      Date:         26042011
 *      Product:      CICS Transaction Gateway for z/OS
 * 
 * Description: The FailOver exit is designed to redirect and retry an ECI request across
 * a group of cloned servers
 *
 * Change History
 * V1.0 - GA version
 * V1.1 - Comment parsing
 * V2.0 - Retry interval and V8 LastServers support
 * V2.1 - Add support for an alternate policy   
 * 
 * See SupportPac documentation for further details on how to configure and use
 * this sample
 * 
 */
package com.ibm.ctg.samples.ha;


import java.util.*;

import com.ibm.ctg.ha.CICSRequestExit;
import com.ibm.ctg.ha.RequestDetails;



public class FailOverExit extends HAExit implements CICSRequestExit {
	//                                             
	public static final String copyright = "\nLicensed Materials - Property of IBM "+
	"CICS SupportPac CA1T" +
	"(c) Copyright IBM Corp. 2008, 2011  All Rights Reserved. " +
	"US Government Users Restricted Rights - Use, duplication or " +
	"disclosure restricted by GSA ADP Schedule Contract with " +
	"IBM Corp.\n";
	//
		

	/**
	 * Constructor called at exit initialisation by Gateway 
	 * 
	 */
	public FailOverExit() throws Exception {

		super(); // invoke super class constructor
		        
	}

    /**
	 * Maps a server alias to a server
	 *  
	 * Input: alias, current time, request map 
	 * Returns: server to be used
	 */
	public synchronized String mapServer(String alias, Date d, 
	      Map<RequestDetails, Object> data) throws NoCICSServersException {
		String[] array = configTable.get(alias); // array of servers to be iterated
		int retry = (Integer) data.get(RequestDetails.RetryCount);
		String server = array[0];// Initialise server to first array member in case all are dirty
      boolean foundClean=false;

      util.debugln("FailOverExit.mapServer(alias=%s,date=%s,retries=%d)",alias,d,retry);

		// Iterate along the array, starting at the retry count until we find a clean server or the end
		for (int i=retry; i < array.length; i++) {
         server = array[i];
			if (isClean(server,d)) {
            foundClean=true;
            util.debugln("CICS server %s is clean (available)",server);
				break; // Server is clean so use it
			} else {
            util.debugln("CICS server %s is dirty (unavailable)",server);
				// Server is dirty so lets go round again
			}
		}
		
      if (!foundClean) {
         // Didn't find any available servers under the current policy
         // Throw NoCICSServersException
         throw new NoCICSServersException("FailOverExit.mapServer() Last CICS server tried="+server);
      }

		
		return server;
	}


}

